#
# Copyright 2015 sodastsai
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from __future__ import unicode_literals, division, absolute_import, print_function
import os
from taskr.contrib.lang import lazy_property
import yaml
from .package import Package
from .installer import Installer, default_installers


class Configuration(object):

    @classmethod
    def auto_discover(cls, source_path=None):
        """
        :rtype: Configuration
        """

        # Find source
        accept_exts = ('yaml', 'yml', 'json')
        dir_path = None
        while not source_path:
            # Go to upper level
            new_dir_path = os.path.split(dir_path)[0] if dir_path else os.getcwd()
            if new_dir_path == dir_path:
                break
            else:
                dir_path = new_dir_path
            # Check file
            for accept_ext in accept_exts:
                source_path = os.path.join(dir_path, 'depot.{}'.format(accept_ext))
                if os.path.exists(source_path):
                    break
                else:
                    source_path = None

        return cls(source_path) if source_path else None

    def __init__(self, config_file_path):
        self._config_file_path = config_file_path
        with open(self._config_file_path, 'r') as f:
            self._config_content = yaml.load(f)
            """:type: dict[str, dict]"""

    @property
    def config_file_path(self):
        return self._config_file_path

    @lazy_property
    def packages(self):
        """
        :rtype: dict[str, list[Package]]
        """
        return {name: list(map(Package, packages))
                for name, packages in self._config_content.get('packages', {}).items()}

    @lazy_property
    def installers(self):
        """
        :rtype: list[Installer]
        """

        # Get installers by merging default and user defined ones
        installers = {installer.name: installer for installer in default_installers}
        installers.update({installer.name: installer for installer in
                           map(lambda x: Installer(x[0], **x[1]), self._config_content.get('installers', {}).items())})

        # Feed installer packages
        for installer in installers.values():
            installer.packages = self.packages.get(installer.name, [])

        # filter out unavailable one and sort by os type
        return sorted(filter(lambda _installer: _installer.available, installers.values()),
                      key=lambda _installer: _installer.os,
                      reverse=True)

    @property
    def commands(self):
        # Traversal installers
        for installer in self.installers:
            commands = []
            multi_install_packages = []
            # Traversal packages
            for package in installer:
                if installer.multiple and not package.test:
                    # Multiple-install directly
                    multi_install_packages.append(package.name)
                else:
                    # Install one-by-one (including test)
                    command = installer.syntax.format(installer.name, package.name)
                    if package.test:
                        command = '{} 1>/dev/null 2>&1 || {}'.format(package.test, command)
                    commands.append(command)
            # Collect multuple-install
            if multi_install_packages:
                commands.append(installer.syntax.format(installer.name, ' '.join(multi_install_packages)))
            # Append sudo
            if installer.sudo:
                commands = map(lambda c: 'sudo {}'.format(c), commands)

            for command in commands:
                yield command



#!/usr/bin/env python

#
# Copyright 2015 sodastsai
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from __future__ import unicode_literals, division, absolute_import, print_function
import os
from depot_pm import __version__ as depot_pm_version
from taskr import task, console
import yaml
from depot_pm.configuration import Configuration


@task
@task.set_argument('package_file', help='path of package file', nargs='?')
@task.set_argument('--verbose', '-v', dest='verbose', action='store_true')
@task.set_argument('--dry-run', '-d', dest='dry_run', action='store_true')
def install(package_file=None, verbose=False, dry_run=False):
    try:
        configuration = Configuration.auto_discover(package_file)
    except yaml.YAMLError as e:
        if verbose:
            console.error('Cannot parse package file: {}'.format(e))
        task.exit(2)
        return  # suppress pycharm warning

    if configuration:
        if verbose:
            console.info('resolved package file: {}'.format(configuration.config_file_path))
    else:
        if verbose:
            console.error('Cannot find package file.')
        task.exit(1)
        return  # suppress pycharm warning

    # Go
    if dry_run and verbose:
        console.info('dry run: depot-pm won\'t perfom following commands.')

    for command in configuration.commands:
        if verbose:
            console.info('Execute command: {}'.format(command))
        if not dry_run:
            os.system(command)
        else:
            console.show(command)

    if verbose:
        console.success('done')


@task
def version():
    print('depot-pm {}'.format(depot_pm_version))


if __name__ == '__main__':
    task.should_raise_exceptions = True
    task.dispatch()



#
# Copyright 2015 sodastsai
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from __future__ import unicode_literals, division, absolute_import, print_function
from taskr.contrib.system import has_command


class Installer(object):

    def __init__(self, installer_name, **kwargs):
        """
        :type installer_name: str
        """
        self.sudo = kwargs.get('sudo', False)
        """:type: bool"""
        self.os = kwargs.get('os', False)
        """:type: bool"""
        self.multiple = kwargs.get('multiple', True)
        """:type: bool"""
        self.syntax = kwargs.get('syntax', '{} install {}')
        """:type: str"""
        self.command = kwargs.get('command', installer_name)
        """:type: str"""
        self.name = installer_name
        """:type: str"""
        self.packages = kwargs.get('packages', [])
        """:type: list[depot_pm.package.Package]"""

    def __repr__(self):
        return '<{}: {}>'.format(self.__class__.__name__, self.name)

    def __iter__(self):
        """
        :rtype: depot_pm.package.Package
        """
        for package in self.packages:
            yield package

    @property
    def available(self):
        return has_command(self.command) and self.packages


default_installers = (
    Installer('pip'),
    Installer('pip3'),
    Installer('gem'),
    Installer('brew', os=True),
    Installer('yum', os=True, sudo=True, syntax='{} install -y {}'),
)



#!/usr/bin/env python

#
# Copyright 2015 sodastsai
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from __future__ import unicode_literals, division, absolute_import, print_function
import os
from setuptools import setup, find_packages

from depot_pm import __version__

with open(os.path.join(os.path.dirname(__file__), 'README.md'), 'r') as f:
    long_description = f.read()

setup(name='depot-pm',
      version=__version__,
      url='https://github.com/sodastsai/depot-pm',
      license='Apache License 2.0',
      author='sodas',
      author_email='sodas2002@gmail.com',
      description='depot package manager helps you manage pacakges',
      long_description=long_description,

      packages=find_packages(),
      install_requires=[
          'PyYAML>=3.08',
          'taskr>=0.2.6',
      ],

      entry_points={
          'console_scripts': [
              'depot-pm = depot_pm.main:task.dispatch',
          ],
      },

      classifiers=[
          'Development Status :: 3 - Alpha',
          'Environment :: Console',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: Apache Software License',
          'Operating System :: MacOS :: MacOS X',
          'Operating System :: POSIX :: Linux',
          'Operating System :: Unix',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.3',
          'Programming Language :: Python :: 3.4',
          'Topic :: Software Development',
          'Topic :: Utilities',
      ])
